#! /home/klotz/tkaudace/bin/audace
#
# astrometry.tcl --
#
# Interface visuelle d'astrometrie pour la plateforme Audace.
#
# (C) 1999 Denis Marchais, Alain Klotz 
#

#############################################################################
# Visual Tcl v1.20 Project
#

#################################
# GLOBAL VARIABLES
#
global gCmdLine; 
global mBinning; 
global mContinu; 
global mExposure; 
global mFilename; 
global mIndex; 
global mIndexValue; 
global widget; 
global path;
global hCanvas;
global Dragging;
global hBox;
global picture;
global box;
#--- caption : associative array pour les textes
global caption;

proc measure_astrom { } {
   global filename
   global caption

   load filename

}

####################################################################
# chargement d'une image par appui sur le bouton LOAD              #
####################################################################
proc LoadCmd { } {
   global filename
   global gCmdLine
   global num_buffer;
   global num_visu;
   global caption

   set filename2 [tk_getOpenFile]
   if { [string compare $filename2 ""] != 0 } {
      if { [string compare [set result [buf$num_buffer(1) load $filename2]] ""] == 0 } {
         set filename $filename2
         wm title .top17 "$caption(astrometry) - $filename"
         visu$num_visu(1) disp
      } else {
         set gCmdLine $result
      }
   }
}

####################################################################
# Menu de l'analyse                                                #
####################################################################
proc analyse_astrom { } {
   global filename
   global caption

   #--- charge une image en memoire
   LoadCmd
   #--- cherche si l'image est calibree
   set result_calib [ calibration_astrom short ]
   # Affiche une fenetre de choix

    #=================================================#
    #=== on definit une nouvelle toplevel window   ===#
    #=================================================#
    set base .top10
    #--- creating widget
    toplevel $base -class Toplevel \
        -background #ba6b8b 
    wm focusmodel $base passive
    wm geometry $base 400x250+10+200
    wm maxsize $base 1024 768
    wm minsize $base 631 453
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "$caption(astrometry) : $caption(analyse) $filename"
    bind $base <Destroy> { destroy .top10 }

    #=================================================#
    #=== on cree les widgets de la toplevel window ===#
    #=================================================#
    #--- Main frame
    frame $base.fraa \
       -borderwidth 1 -height 75 -relief groove -width 125 
    pack $base.fraa \
      -in $base -anchor center -expand 1 -fill both -side top 
    if { [string compare $result_calib "no"] == 0 } {
       #--- label : "this image is not calibrated"
       set chaine $caption(astrom1b)
       label $base.fraa.lab17 \
          -borderwidth 1 -text $chaine -font systemfixed 
       pack $base.fraa.lab17 \
          -in $base.fraa -anchor center -expand 1 -fill both -side top
       button $base.fraa.but1 -text $caption(calibration_cat) -command { calibration_astrom long ; destroy .top10 ; return} -width 12
       pack $base.fraa.but1 -anchor center -side top -fill x -padx 4 -pady 4 \
          -in $base.fraa -anchor center -expand 1 -fill both -side top
       button $base.fraa.but2 -text $caption(calibration_img) -command { } -width 12
       pack $base.fraa.but2 -anchor center -side top -fill x -padx 4 -pady 4 \
          -in $base.fraa -anchor center -expand 1 -fill both -side top
    } else {
       #--- label : "this image is calibrated"
       set chaine $caption(astrom1)
       label $base.fraa.lab17 \
          -borderwidth 1 -text $chaine -font systemfixed 
       pack $base.fraa.lab17 \
          -in $base.fraa -anchor center -expand 1 -fill both -side top
       button $base.fraa.but0 -text $caption(measure) -command { destroy .top10 ; measure_astrom ; return } -width 12
       pack $base.fraa.but0 -anchor center -side top -fill x -padx 4 -pady 4 \
          -in $base.fraa -anchor center -expand 1 -fill both -side top
       button $base.fraa.but1 -text $caption(calibration_cat) -command { calibration_astrom long ; destroy .top10 ; return} -width 12
       pack $base.fraa.but1 -anchor center -side top -fill x -padx 4 -pady 4 \
          -in $base.fraa -anchor center -expand 1 -fill both -side top
       button $base.fraa.but2 -text $caption(calibration_img) -command { } -width 12
       pack $base.fraa.but2 -anchor center -side top -fill x -padx 4 -pady 4 \
          -in $base.fraa -anchor center -expand 1 -fill both -side top
    }
    #-- button (exit) ---
    button $base.fraa.butexit -text $caption(exit) -command {destroy .top10; return} -width 12
    pack $base.fraa.butexit -anchor center -side top -fill x -padx 4 -pady 4 \
      -in $base.fraa -anchor center -expand 1 -fill both -side top
    focus .top10

}

####################################################################
# calibration astrometrique automatique                            #
####################################################################
proc calibration_astrom { analyse } {
   global filename
   global caption
   global astrom_keys1
   global astrom_values
   global astrom_values1
   global astrom_comments
   global astrom_units
   global rep_cat
   global name_cat
   global name_cat0
   global kappa
   global border

   set pi [expr 2*asin(1.0)]
   set deg2rad [expr $pi/180.0]
   set rad2deg [expr 180.0/$pi]


    #--- init variables ---#
    set list_header [buf1 getkwds]
    #
    set astrom_keys1 [list CRPIX1 CRPIX2 CRVAL1 CRVAL2 FOCLEN PIXSIZE1 PIXSIZE2]
    set astrom_keys11 [list CRPIX1 CRPIX2 RA DEC FOCLEN PIXSIZE1 PIXSIZE2]
    set astrom_keys2 [list CRPIX1 CRPIX2 CRVAL1 CRVAL2 CDELT1 CDELT2 CROTA2 ]
    set astrom_keys3 [list CRPIX1 CRPIX2 CRVAL1 CRVAL2 CD1_1 CD1_2 CD2_1 CD2_2]
    #
    set astrom_keys [list CRPIX1 CRPIX2 CRVAL1 CRVAL2 FOCLEN PIXSIZE1 PIXSIZE2 CRVAL1 CRVAL2 CDELT1 CDELT2 CROTA2 CD1_1 CD1_2 CD2_1 CD2_2]
    foreach key $astrom_keys {
       set astrom_values($key) ""
    }
    foreach key $astrom_keys1 {
       set astrom_values1($key) ""
    }
    #
    set texte [expr 0.5*[lindex [buf1 getkwd NAXIS1] 1] ]
    set xcrpix1 "$texte"
    set texte [expr 0.5*[lindex [buf1 getkwd NAXIS2] 1] ]
    set xcrpix2 "$texte"
    set astrom_values1(CRPIX1) $xcrpix1
    set astrom_values1(CRPIX2) $xcrpix2
    #
    set astrom_units [list pix pix deg deg m um um] 
    set astrom_comments [list "reference pixel for naxis1" "reference pixel for naxis2" "Right ascension" "Declination" "Focal length" "Pixel size along naxis1" "Pixel size along naxis2"] 

    #--- test if the image is already calibrated ---#
    #--- search for the new FITS keywords ---#
    set astrom_found3 {}
    foreach key $astrom_keys3 {
       set index_key [lsearch -exact $list_header $key]
       if {$index_key == -1} {
          set a [linsert $astrom_found3 end no]
       } else {
          set a [linsert $astrom_found3 end yes]
       }
       set astrom_found3 $a
       unset a
    }

    #--- test if the image is already calibrated ---#
    #--- search for the old FITS keywords ---#
    set astrom_found2 {}
    foreach key $astrom_keys2 {
       set index_key [lsearch -exact $list_header $key]
       if {$index_key == -1} {
          set a [linsert $astrom_found2 end no]
       } else {
          set a [linsert $astrom_found2 end yes]
       }
       set astrom_found2 $a
       unset a
    }

    #--- search for the FITS keywords necessary for calibration ---#
    set astrom_found1 {}
    foreach key $astrom_keys1 {
       set index_key [lsearch -exact $list_header $key]
       if {$index_key == -1} {
          set a [linsert $astrom_found1 end no]
       } else {
          set a [linsert $astrom_found1 end yes]
       }
       set astrom_found1 $a
       unset a
    }
    set astrom_found11 {}
    foreach key $astrom_keys11 {
       set index_key [lsearch -exact $list_header $key]
       if {$index_key == -1} {
          set a [linsert $astrom_found11 end no]
       } else {
          set a [linsert $astrom_found11 end yes]
       }
       set astrom_found11 $a
       unset a
    }

    #--- determine the type of calibration (no old new) ---#
    set result_calib no
    #--- is image is calibrated with new FITS keywords ? ---#
    if {[lsearch -exact $astrom_found3 no] == 0} {
       #--- no ! then, is image is calibrated with old FITS keywords ? ---#
       if {[lsearch -exact $astrom_found2 no] == 0} {
          #--- no ! the image is not calibrated ---#
          set result_calib no
       } else {
          set result_calib old
          set astrom_keys0 $astrom_keys2
       }
    } else {
       set result_calib new
       set astrom_keys0 $astrom_keys3
    }

    if {$analyse == "short"} { return $result_calib }

    #--- in the case of a calibrated image ---#
    #--- convert the keywords to FITS keywords necessary for calibration ---#
    if {[string compare $result_calib new ] == 0 } {
        #--- read the 'new' values
        foreach key $astrom_keys3 {
           set card [buf1 getkwd $key]
           set astrom_values3($key) [lindex $card 1]
        }
        #--- convert to the 'old' values
        set astrom_values2(CRPIX1) $astrom_values3(CRPIX1)
        set astrom_values2(CRPIX2) $astrom_values3(CRPIX2)
        set astrom_values2(CRVAL1) $astrom_values3(CRVAL1)
        set astrom_values2(CRVAL2) $astrom_values3(CRVAL2)
        set num [expr $astrom_values3(CD1_2)-$astrom_values3(CD1_2)]
        set den [expr $astrom_values3(CD1_1)+$astrom_values3(CD2_2)]
        set astrom_values2(CROTA2) [expr $rad2deg*atan2($num,$den)]
        set num [expr sqrt($astrom_values3(CD1_1)*$astrom_values3(CD1_1)+$astrom_values3(CD2_1)*$astrom_values3(CD2_1))]
        set signe [expr $astrom_values3(CD1_1)*cos($astrom_values2(CROTA2))]
        set signe [expr $signe/abs($signe)]
        set astrom_values2(CDELT1) [expr $num*$signe]
        set num [expr sqrt($astrom_values3(CD1_2)*$astrom_values3(CD1_2)+$astrom_values3(CD2_2)*$astrom_values3(CD2_2))]
        set signe [expr $astrom_values3(CD2_2)*cos($astrom_values2(CROTA2))]
        set signe [expr $signe/abs($signe)]
        set astrom_values2(CDELT2) [expr $num*$signe]
        set result_calib old
    }
    if {[string compare $result_calib old ] == 0 } {
        set astrom_values1(CRPIX1) $astrom_values2(CRPIX1)
        set astrom_values1(CRPIX2) $astrom_values2(CRPIX2)
        set astrom_values1(CRVAL1) $astrom_values2(CRVAL1)
        set astrom_values1(CRVAL2) $astrom_values2(CRVAL2)
        set tan_cdelt1 [expr tan($deg2rad*$astrom_values2(CDELT1))]
        set tan_cdelt2 [expr tan($deg2rad*$astrom_values2(CDELT2))]
        #
        set key PIXSIZE1
        set index_key [lsearch -exact $list_header $key]
        if {$index_key == -1} {
           set astrom_values1($key) 10
        } else {
           set card [buf1 getkwd $key]
           set astrom_values1($key) [lindex $card 1]
        }
        set key PIXSIZE2
        set index_key [lsearch -exact $list_header $key]
        if {$index_key == -1} {
           set astrom_values1($key) 10
        } else {
           set card [buf1 getkwd $key]
           set astrom_values1($key) [lindex $card 1]
        }
        set astrom_values1(FOCLEN) [expr abs($astrom_values1(PIXSIZE1)*1.e-6/$tan_cdelt1)]
    } elseif {[lsearch -exact [lrange $astrom_found1 2 end] no] != 0} {
        #--- read the 'custom' values
        foreach key $astrom_keys1 {
           set card [buf1 getkwd $key]
           if {[lindex $card 1] != "" } {
              set astrom_values1($key) [lindex $card 1]
           }
        }
    } elseif {[lsearch -exact [lrange $astrom_found11 2 end] no] != 0} {
        #--- read the 'custom' values
        foreach key $astrom_keys11 {
           set card [buf1 getkwd $key]
           if {$key == "RA"} { set key CRVAL1 }
           if {$key == "DEC"} { set key CRVAL2 }
           if {[lindex $card 1] != "" } {
              set astrom_values1($key) [lindex $card 1]
           }
        }
    }

    #=================================================#
    #=== on definit une nouvelle toplevel window   ===#
    #=================================================#
    set base .top16
    #--- creating widget
    toplevel $base -class Toplevel \
        -background #ba6b8b 
    wm focusmodel $base passive
    wm geometry $base 640x480+10+200
    wm maxsize $base 1024 768
    wm minsize $base 631 453
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "$caption(astrometry) : $caption(calibration) $filename"
    bind $base <Destroy> {destroy .top16}

    #=================================================#
    #=== on cree les widgets de la toplevel window ===#
    #=================================================#

    #--- in the case of a calibrated image ---#
    #--- convert the keywords to FITS keywords necessary for calibration ---#


    #--- case : the image was already calibrated 
    if {[string compare $result_calib no ] != 0 } {
       #--- texte d'explications
       #--- cree les widgets
       frame $base.fra0 \
          -borderwidth 1 -height 75 -relief groove -width 125 
       set chaine $caption(astrom1)
       label $base.fra0.lab0 \
          -borderwidth 1 -text $chaine
       #--- setting geometry
       pack $base.fra0 \
          -in $base -anchor center -expand 0 -fill x -side top 
       pack $base.fra0.lab0 \
          -in $base.fra0 -anchor center -expand 0 -fill x -side left
    }

    #--- case : ask for keywords values necessary for calibration ---#
       #--- texte d'explications
       #--- cree les widgets
       frame $base.fra00 \
          -borderwidth 1 -height 75 -relief groove -width 125 
       set chaine $caption(astrom2)
       label $base.fra00.lab0 \
          -borderwidth 1 -text $chaine
       #--- setting geometry
       pack $base.fra00 \
          -in $base -anchor center -expand 0 -fill x -side top 
       pack $base.fra00.lab0 \
          -in $base.fra00 -anchor center -expand 0 -fill x -side left
       #--- textes des mots cles
       set indice 0
       foreach key $astrom_keys1 {
          #--- chaine = Keyword + Value + Unit ---#
          set chaine $key
          set longs [string length $key]
          set texte [string range "            " 0 [expr 8-$longs]]
          append chaine $texte
          set texte [lindex $astrom_values1($key) 0]
          if { $texte != "" } {          
             set texte [format "%10f" [lindex $astrom_values1($key) 0]]
          }
          set chaine2 " $texte"
          set texte " "
          append chaine2 $texte
          set texte [lindex $astrom_units $indice]
          append chaine2 " ($texte)."
          if {$astrom_values1($key) != ""} {
             set texte [lindex $astrom_units $indice]
             append chaine2 " $caption(by_default)."
          }

          #--- cree les widgets
          incr indice
          frame $base.fra0$indice \
             -borderwidth 1 -height 75 -relief groove -width 125 
          label $base.fra0$indice.lab1 \
             -borderwidth 1 -text $chaine -font systemfixed 
          entry $base.fra0$indice.ent1 \
             -font {{Arial} 8 bold} -textvariable astrom_values($key) -width 20 
          label $base.fra0$indice.lab2 \
             -borderwidth 1 -text $chaine2 -font systemfixed 
          #--- setting geometry
          pack $base.fra0$indice \
             -in $base -anchor center -expand 0 -fill x -side top 
          pack $base.fra0$indice.lab1 \
             -in $base.fra0$indice -anchor center -expand 0 -fill x -side left
          pack $base.fra0$indice.ent1 \
             -in $base.fra0$indice -anchor center -expand 0 -fill x -side left
          pack $base.fra0$indice.lab2 \
             -in $base.fra0$indice -anchor center -expand 0 -fill x -side left
       }

    #--- create the entry for path of Cdrom ---#
    set rep_cat f:\\
    set name_cat Microcat
    frame $base.fraaa1 \
       -borderwidth 1 -height 75 -relief groove -width 125 
    pack $base.fraaa1 \
       -in $base -anchor center -expand 0 -fill x -side top 
    #
    set chaine " $caption(rep_microcat) "
    label $base.fraaa1.lab1 \
       -borderwidth 1 -text $chaine
    pack $base.fraaa1.lab1 \
       -in $base.fraaa1 -anchor center -expand 0 -fill x -side left
    #
    tk_optionMenu $base.options1 name_cat0 Microcat USNO
    pack $base.options1 -side left -in $base.fraaa1
    #-padx 25 -pady 25
    #
    entry $base.fraaa1.ent1 \
       -font {{Arial} 8 bold} -textvariable astrom_values(rep_microcat) -width 20 
    pack $base.fraaa1.ent1 \
       -in $base.fraaa1 -anchor center -expand 0 -fill x -side left
    #
    set chaine "$rep_cat ($caption(by_default))."
    label $base.fraaa1.lab2 \
       -borderwidth 1 -text $chaine
    pack $base.fraaa1.lab2 \
       -in $base.fraaa1 -anchor center -expand 0 -fill x -side left

    #--- create buttom to start the calibration ---#
    frame $base.fraaa \
       -borderwidth 1 -height 75 -relief groove -width 125 
    pack $base.fraaa \
       -in $base -anchor center -expand 0 -fill x -side top 
    #
    button $base.fraaa.but1 -text $caption(calibration) -command {calibration_astrom2; destroy .top16 }
    pack $base.fraaa.but1 -side left -fill x -padx 4 -pady 4
    #
    set chaine "   $caption(border) "
    label $base.fraaa.lab2 \
       -borderwidth 1 -text $chaine
    pack $base.fraaa.lab2 \
       -in $base.fraaa -anchor center -expand 0 -fill x -side left
    tk_optionMenu $base.options2 border 0 10 20 50
    pack $base.options2 -side left -in $base.fraaa
    set chaine "   $caption(kappa) "
    label $base.fraaa.lab3 \
       -borderwidth 1 -text $chaine
    pack $base.fraaa.lab3 \
       -in $base.fraaa -anchor center -expand 0 -fill x -side left
    tk_optionMenu $base.options3 kappa 3 5 10
    pack $base.options3 -side left -in $base.fraaa

    #--- create buttom to quit ---#
    frame $base.fraaaa \
       -borderwidth 1 -height 75 -relief groove -width 125 
    pack $base.fraaaa \
       -in $base -anchor center -expand 1 -fill both -side top 
    button $base.fraaaa.but1 -text $caption(exit_calibration) -command {destroy .top16 }
    pack $base.fraaaa.but1 -side left -fill both -padx 4 -pady 4

}


#########################################################################
# Module appele en interne par la calibration astrometrique automatique #
#########################################################################
proc calibration_astrom2 {} {
   global filename
   global caption
   global astrom_keys1
   global astrom_values1
   global astrom_values
   global astrom_comments
   global astrom_units
   global rep_cat
   global name_cat
   global name_cat0
   global kappa
   global border

   #
   set out toto

   if { $name_cat0 == "Microcat" } { set name_cat USNOCOMP } 
   if { $name_cat0 == "USNO" } { set name_cat USNO } 

   if {$astrom_values(rep_microcat) == ""} {
      set $astrom_values(rep_microcat) $rep_cat
   }

   #--- analyse l'entete ---#
   set h [lindex $astrom_values(CRVAL1) 0]
   if {$h != ""} {
      set m [lindex $astrom_values(CRVAL1) 1]
      set s [lindex $astrom_values(CRVAL1) 2]
      if {[string compare $m ""] == 0 && [string compare $s ""] == 0} {
         #--- field in degrees
         set ra $h
      } else {
         #--- field in h m s
         if {$m == ""} { set m 0 }
         if {$s == ""} { set s 0 }
         set ra [expr ($h+$m/60.+$s/3600.)*15.]
      }
      set astrom_values(CRVAL1) $ra
   }
   set d [lindex $astrom_values(CRVAL2) 0]
   if {$d != ""} {
      set m [lindex $astrom_values(CRVAL2) 1]
      if {$m == ""} { set m 0 }
      set s [lindex $astrom_values(CRVAL2) 2]
      if {$s == ""} { set s 0 }
      set dec [expr ($d+$m/60.+$s/3600.)]
      set astrom_values(CRVAL2) $dec
   }
   #
   set chaine ""
   set index 0
   foreach key $astrom_keys1 {
      set value_exist no
      if { [string compare $astrom_values($key) ""] != 0 } {
         append chaine $key\n
         append chaine $astrom_values($key)\n
         append chaine double\n
         append chaine [lindex $astrom_comments $index]\n
         append chaine [lindex $astrom_units $index]\n
         set value_exist yes
         incr index
      } elseif { [string compare $astrom_values1($key) ""] != 0 } {
         append chaine $key\n
         append chaine $astrom_values1($key)\n
         append chaine double\n
         append chaine [lindex $astrom_comments $index]\n
         append chaine [lindex $astrom_units $index]\n
         set value_exist yes
         incr index
      }
      if { [string compare $value_exist no] == 0 } {
            #--- this field was not filled ---#
            destroy .top15
            set base .top155
            #--- creating widget
            toplevel $base -class Toplevel \
               -background #ba6b8b 
            wm focusmodel $base passive
            wm geometry $base 400x100+10+200
            wm maxsize $base 400 100
            wm minsize $base 400 100
            wm overrideredirect $base 0
            wm resizable $base 1 1
            wm deiconify $base
            wm title $base "$caption(astrometry) : $caption(astrom3) $filename"
            bind $base <Destroy> {destroy .top155}
            #--- cree les widgets
            frame $base.fraa \
               -borderwidth 1 -height 75 -relief groove -width 125 
            set chaine "$caption(astrom6)\n\n $key"
            label $base.fraa.lab17 \
               -borderwidth 1 -text $chaine -font systemfixed 
            button $base.fraa.butexit -text $caption(exit) -command {destroy .top155; return} -width 12
            #--- setting geometry
            pack $base.fraa \
               -in $base -anchor center -expand 1 -fill both -side top 
            pack $base.fraa.lab17 \
               -in $base.fraa -anchor center -expand 1 -fill both -side top
            pack $base.fraa.butexit -anchor center -side top -fill x -padx 4 -pady 4 \
               -in $base.fraa -anchor center -expand 1 -fill both -side top
            focus .top155
      }
   }

   #
   set rep [string range $filename 0 [string last / $filename]]
   set namext [string range $filename [expr 1+[string last / $filename]] end]
   set name [string range $namext 0 [expr [string last . $namext]-1] ]
   set ext [string range $namext [string last . $namext] end]
   #
   if {$index != 0} {
      if { [catch {open "$rep$out.hdr" w} output] } {
         set a 0
      } else {
         puts $output $chaine
         close $output
      }
   }

    #=================================================#
    #=== on definit une nouvelle toplevel window   ===#
    #=================================================#
    set base .top15
    #--- creating widget
    toplevel $base -class Toplevel \
        -background #ba6b8b 
    wm focusmodel $base passive
    wm geometry $base 400x100+10+200
    wm maxsize $base 400 100
    wm minsize $base 400 100
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "$caption(astrometry) : $caption(astrom3) $filename"
    #--- cree les widgets
    frame $base.fraa \
       -borderwidth 1 -height 75 -relief groove -width 125 
    set chaine "$filename\n\n$caption(astrom3)"
    label $base.fraa.lab17 \
       -borderwidth 1 -text $chaine -font systemfixed 
    #--- setting geometry
    pack $base.fraa \
       -in $base -anchor center -expand 1 -fill both -side top 
    pack $base.fraa.lab17 \
       -in $base.fraa -anchor center -expand 1 -fill both -side top
    update
   #
   set chaine ""
   append chaine "SET/VAR rep\$ $rep\n"
   append chaine "SET/VAR in\$ $name\n"
   append chaine "SET/VAR out\$ $out\n"
   append chaine "SET/VAR rep_usno\$ $rep_cat\n"
   append chaine "SET/VAR ext\$ $ext\n"
   append chaine "IMA/SERIES rep\$ out\$l . . ext\$ rep\$ . . ext\$ DELETE\n"
   append chaine "IMA/SERIES rep\$ in\$ . . ext\$ rep\$ out\$ . ext\$ STAT objefile=rep\$out\$l nullpixel=-1000 border=$border detect_kappa=$kappa\n"
   if {$index != 0} {
      append chaine "IMA/SERIES rep\$ out\$ . . ext\$ rep\$ out\$ . ext\$ HEADERFITS file=rep\$out\$.hdr\n"
   }
   append chaine "IMA/SERIES rep\$ out\$ . . ext\$ rep\$ out\$ . ext\$ CATCHART astromcatalog=$name_cat path_astromcatalog=rep_usno\$ catafile=rep\$out\$l.fit jpegfile_chart=rep\$out\$1.jpg border=$border\n"
   append chaine "IMA/SERIES rep\$ out\$ . . ext\$ rep\$ out\$ . ext\$ ASTROMETRY\n"
   append chaine "IMA/SERIES rep\$ out\$ . . ext\$ rep\$ out\$ . ext\$ CATCHART astromcatalog=$name_cat path_astromcatalog=rep_usno\$ catafile=rep\$out\$l.fit jpegfile_chart2=rep\$out\$2.jpg\n"
   append chaine "IMA/SERIES rep\$ out\$ . . ext\$ rep\$ out\$ . ext\$ ASTROMETRY\n"
   #.console.log insert end $chaine
      if { [catch {open "$rep$out.tt" w} output] } {
         set a 0
      } else {
         puts $output $chaine
         close $output
      }

   ttscript2 $chaine
   destroy .top15

   set fullname $rep
   append fullname $out$ext
   if { [string compare [set result [buf2 load $fullname]] ""] == 0 } {
   }

   set card [buf2 getkwd CATASTAR]
   set catastar [lindex $card 1]
    #=================================================#
    #=== on definit une nouvelle toplevel window   ===#
    #=================================================#
    set base .top150
    #--- creating widget
    toplevel $base -class Toplevel \
        -background #ba6b8b 
    wm focusmodel $base passive
    wm geometry $base 400x100+10+200
    wm maxsize $base 400 100
    wm minsize $base 400 100
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "$caption(astrometry) : $caption(calib_enb) $filename"
    #--- cree les widgets
    frame $base.fraa \
       -borderwidth 1 -height 75 -relief groove -width 125 
    set chaine "$catastar $caption(nb_star_found)"
    label $base.fraa.lab17 \
       -borderwidth 1 -text $chaine -font systemfixed 
    button $base.fraa.but1 -text $caption(saveas) -command { buf2 save [set filename [tk_getSaveFile]] ; destroy .top150; return} -width 12
    button $base.fraa.butexit -text $caption(exit) -command {destroy .top150; return} -width 12
    #--- setting geometry
    pack $base.fraa \
       -in $base -anchor center -expand 1 -fill both -side top 
    pack $base.fraa.lab17 \
       -in $base.fraa -anchor center -expand 1 -fill both -side top
    pack $base.fraa.but1 -anchor center -side top -fill x -padx 4 -pady 4 \
       -in $base.fraa -anchor center -expand 1 -fill both -side top
    pack $base.fraa.butexit -anchor center -side top -fill x -padx 4 -pady 4 \
       -in $base.fraa -anchor center -expand 1 -fill both -side top
    focus $base
       
}


####################################################################
# Affiche le contenu du header                                     #
####################################################################
proc header_astrom { {base ""} } {
   global filename
    global caption
    set nb_kw_disp 50
    
    #=================================================#
    #=== on definit une nouvelle toplevel window   ===#
    #=================================================#
    if {$base == ""} {
        set base .top18
    }
    if {[winfo exists $base]} {
        wm deiconify $base; return
    }
    #--- creating widget
    toplevel $base -class Toplevel \
        -background #ba6b8b 
    wm focusmodel $base passive
    wm geometry $base 640x480+10+200
    wm maxsize $base 1024 768
    wm minsize $base 631 453
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "$caption(astrometry) : $caption(header) $filename"

    #=================================================#
    #=== on cree les widgets de la toplevel window ===#
    #=================================================#
    #--- cree un frame pour la listbos et la scrollbar
    frame $base.fra18 \
        -background #a7bbda 
    #--- cree la listbos et sa scrollbar
    listbox $base.fra18.3 -font systemfixed -width 80 -height $nb_kw_disp -yscrollcommand {.top18.fra18.scr set}
    scrollbar $base.fra18.scr -orient vertical -command {.top18.fra18.3 yview}
    #--- setting geometry
    pack $base.fra18 \
        -in $base -anchor center -expand 0 -fill x -side bottom 
    pack $base.fra18.scr -side right -pady 4 -padx 4 -fill both
    pack $base.fra18.3 -side left -fill both -pady 4 -padx 4

    
    #================================================#
    #=== on remplit la listbox avec l'entete FITS ===#
    #================================================#
    $base.fra18.3 delete 0 end
    foreach kw [buf1 getkwds] {
       set chaine ""
       set texte [lindex [buf1 getkwd $kw] 0]
       append chaine $texte
       set longs [string length $texte]
       set texte [string range "            " 0 [expr 8-$longs]]
       append chaine $texte
       if { [string compare [lindex [buf1 getkwd $kw] 2] string] == 0 } {
          append chaine '
       }
       set texte [lindex [buf1 getkwd $kw] 1]
       append chaine $texte
       if { [string compare [lindex [buf1 getkwd $kw] 2] string] == 0 } {
          append chaine '
       }
       set texte [lindex [buf1 getkwd $kw] 3]
       if { [string compare texte ""] != 0 } {
          append chaine " / "
          append chaine $texte
       }       
       $base.fra18.3 insert end $chaine
    }
    bind $base <Destroy> {destroy .top18}
}

