#! /home/klotz/tkaudace/bin/audace
#
# astrometry.tcl --
#
# Interface visuelle d'astrometrie pour la plateforme Audace.
#
# (C) 1999 Denis Marchais, Alain Klotz 
#

#############################################################################
# Visual Tcl v1.20 Project
#

#################################
# GLOBAL VARIABLES
#
global gCmdLine; 
global mBinning; 
global mContinu; 
global mExposure; 
global mFilename; 
global mIndex; 
global mIndexValue; 
global widget; 
global path;
global hCanvas;
global Dragging;
global hBox;
global picture;
global box;
#--- caption : associative array pour les textes
global caption;

set picture(orgx) 10
set picture(orgy) 10
set Dragging 0
set hBox 0


####################################################################
####################################################################
# Procedures d'initialisation du programme.                        #
####################################################################
####################################################################

####################################################################
# initialisation de variables diverses.                            #
####################################################################
proc init {argc argv} {
  global caption
  global filename

  set filename ""
  upvar #0 mBinning umBinning
  set umBinning 1

  #======================================================#
  #=== Definition des mots a ecrire dans les captions ===#
  #======================================================#
  #--- selection du langage
  set langage french
  #set langage english
  #--- definition des contenus des captions
  if {[string compare $langage english] ==0 } {
     set caption(astrometry) "Astrometry"
     set caption(analyse) "Analyse an image"
     set caption(header) "Header"
     set caption(calibration) "Calibrate"
     set caption(measure) "Measure coordinates"
     set caption(calibration_cat) "Calibrate with a catalog"
     set caption(calibration_img) "Calibrate with an other image"
     set caption(load) "Load"
     set caption(saveas) "Save as ..."
     set caption(exit) "Exit"
     set caption(astrom1) "This image seems calibrated."
     set caption(astrom1b) "This image seems not be calibrated."
     set caption(astrom2) "List of mendatory keywords to complete for the FITS header before calibration."
     set caption(astrom3) "Calibration in process..."
     set caption(astrom4) "should be the best"
     set caption(astrom5) "micrometers"
     set caption(astrom6) "You didn't filled the field of"
     set caption(by_default) "Default value"
     set caption(exit_calibration) "Don't perform calibration"
     set caption(rep_microcat) "Path to acces catalog "
     set caption(nb_star_found) "stars matched with the catalog"
     set caption(calib_enb) "End of calibration "
     set caption(kappa) "Detectivity (kappa)"
     set caption(border) "Border (percent)"
  }
  if {[string compare $langage french] ==0 } {
     set caption(astrometry) "Astrometrie"
     set caption(analyse) "Analyse une image"
     set caption(header) "Entete"
     set caption(calibration) "Calibrer"
     set caption(measure) "Mesurer des coordonnees"
     set caption(calibration_cat) "Calibrer avec un catalogue"
     set caption(calibration_img) "Calibrer avec une autre image"
     set caption(load) "Charger"
     set caption(saveas) "Enregistrer sous ..."
     set caption(exit) "Quitter"
     set caption(astrom1) "Cette image semble calibree."
     set caption(astrom1b) "Cette image ne semble pas calibree."
     set caption(astrom2) "Liste de mots cle necessaires pour completer l'entete FITS avant la calibration."
     set caption(astrom3) "Calibration en cours..."
     set caption(astrom4) "devrait etre optimal"
     set caption(astrom5) "micrometres"
     set caption(astrom6) "Vous n'avez pas remplit le champ de"
     set caption(by_default) "Valeur par defaut"
     set caption(exit_calibration) "Ne pas calibrer"
     set caption(rep_microcat) "Dossier pour acceder au catalogue "
     set caption(nb_star_found) "etoiles appariees avec le catalogue"
     set caption(calib_enb) "Calibration terminee "
     set caption(kappa) "Detectivite (kappa)"
     set caption(border) "Bordure (pourcent)"
  }

  source astrom0.tcl

}


####################################################################
####################################################################
proc {main} {argc argv} {
  global hCanvas
  global picture
  global num_buffer;
  global num_visu;
  global caption

  #--- reserve un buffer dans la memoire pour charger une image ---#
  set num_buffer(1) [buf::create]
  set num_buffer(2) [buf::create]

  #--- cree le lien d'affichage entre le buffer $num_buffer(1) ---#
  #--- et image1 du canvas d'affichage qui sera cree ulterieurement ---#
  set num_visu(1) [visu::create $num_buffer(1) 1 ]

  catch {
    #--- on va creer ici la zone d'image qui va etre affichable dans le canvas can18 ---#
    set hCanvas .top17.can18
    $hCanvas create image $picture(orgx) $picture(orgy) -image image1 -anchor nw -tag img1
  } msg

  #--- on detruira la toplevel du programme si la commande exit est actionnee ---#
  bind .top17 <Destroy> {exit}

  #--- charge les librairies et les proc utiles ---#
  catch {source libs.tcl}
  catch {source console.tcl}
  catch {source init.tcl}
  cd ..

}


####################################################################
# Fonction generique pour creer une nouvelle fenetre facilement.   #
####################################################################
proc {Window} {args} {
    global vTcl
    set cmd [lindex $args 0]
    set name [lindex $args 1]
    set newname [lindex $args 2]
    set rest [lrange $args 3 end]
    if {$name == "" || $cmd == ""} {return}
    if {$newname == ""} {
        set newname $name
    }
    set exists [winfo exists $newname]
    switch $cmd {
        show {
            if {$exists == "1" && $name != "."} {wm deiconify $name; return}
            if {[info procs vTclWindow(pre)$name] != ""} {
                eval "vTclWindow(pre)$name $newname $rest"
            }
            if {[info procs vTclWindow$name] != ""} {
                eval "vTclWindow$name $newname $rest"
            }
            if {[info procs vTclWindow(post)$name] != ""} {
                eval "vTclWindow(post)$name $newname $rest"
            }
        }
        hide    { if $exists {wm withdraw $newname; return} }
        iconify { if $exists {wm iconify $newname; return} }
        destroy { if $exists {destroy $newname; return} }
    }
}


####################################################################
# definition de la toplevel window .                               #
####################################################################
# Elle ne contient rien du tout et ne sera pas affichee            #
####################################################################
proc vTclWindow. {base} {
    if {$base == ""} {
        set base .
    }
    #=======================================#
    #=== on definit la toplevel window   ===#
    #=======================================#
    #--- creating widget
    wm focusmodel $base passive
    wm geometry $base 200x200+0+0
    wm maxsize $base 1024 753
    wm minsize $base 104 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm withdraw $base
    wm title $base "vt"
}


####################################################################
# definition de la toplevel window .top17                          #
####################################################################
# Elle contient la ligne de commande, les boutons, la visu         #
####################################################################
proc vTclWindow.top17 {base} {
  global caption
    if {$base == ""} {
        set base .top17
    }
    if {[winfo exists $base]} {
        wm deiconify $base; return
    }

    #=======================================#
    #=== on definit la toplevel window   ===#
    #=======================================#
    #--- creating widget
    toplevel $base -class Toplevel \
        -background #ba6b8b 
    wm focusmodel $base passive
    wm geometry $base 640x480+10+200
    wm maxsize $base 1024 768
    wm minsize $base 631 453
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base $caption(astrometry)

    #=================================================#
    #=== on cree les widgets de la toplevel window ===#
    #=================================================#
    #--- cree un frame pour la ligne de commande
    frame $base.fra18 \
        -background #a7bbda -borderwidth 2 -cursor arrow -height 46 \
        -relief groove -width 125 
    #--- cree un frame pour y mettre les boutons
    frame $base.fra19 \
        -background #a7bbda -borderwidth 2 -cursor arrow -height 46 \
        -relief groove -width 125 
    #--- cree le canvas pour l'affichage des images
    canvas $base.can18 \
        -background #c0cfb1 -borderwidth 1 -height 265 -relief groove \
        -width 379 

    #======================================#
    #=== on garnit la ligne de commande ===#
    #======================================#
    #--- creating widget
    entry $base.fra18.ent30 \
        -font {{Arial}  8 bold} -textvariable gCmdLine 
    bind $base.fra18.ent30 <Key-Return> {
        history add "$gCmdLine"
        set resultat [eval $gCmdLine]
        if { [string compare $resultat ""] != 0 } {
            .console.log insert end "> $resultat\n"
        }
        set gCmdLine ""        
    }
    bind $base.fra18.ent30 <Key-Up> {
        set gCmdLine [history before]
        .top17.fra18.ent30 icursor end
    }
    bind $base.fra18.ent30 <Key-Down> {
        set gCmdLine [history after]
        .top17.fra18.ent30 icursor end
    }
    bind $base.fra18.ent30 <Key-Escape> {
        set gCmdLine [history synchro]
    }
    # --- setting geometry
    pack $base.fra18 \
        -in .top17 -anchor center -expand 0 -fill x -side bottom 
    place $base.fra18.ent30 \
        -x 12 -y 12 -width 229 -anchor nw -bordermode ignore 

    #=========================================#
    #=== on garnit les boutons de commande ===#
    #=========================================#
    #--- creating widget
    button $base.fra19.but0 -text $caption(analyse) -command { analyse_astrom }
    #button $base.fra19.but1 -text $caption(load) -command { LoadCmd }
    #button $base.fra19.but2 -text $caption(header) -command {header_astrom }
    #button $base.fra19.but3 -text $caption(calibration) -command {calibration_astrom long }
    button $base.fra19.butexit -text $caption(exit) -command {exit} -width 12
    # --- setting geometry
    pack $base.fra19 \
        -in .top17 -anchor center -expand 0 -fill x -side bottom 
    pack $base.fra19.but0 -side left -fill x -padx 4 -pady 4
    #pack $base.fra19.but1 -side left -fill x -padx 4 -pady 4
    #pack $base.fra19.but2 -side left -fill x -padx 4 -pady 4
    #pack $base.fra19.but3 -side left -fill x -padx 4 -pady 4
    pack $base.fra19.butexit -side left -fill x -padx 4 -pady 4

    #=======================================#
    #=== on garnit le canvas d'affichage ===#
    #=======================================#
    # --- setting geometry
    pack $base.can18 \
        -in .top17 -anchor center -expand 1 -fill both -side right 
}


####################################################################
####################################################################
## Debut de l'execution du script TCL.                             #
####################################################################
####################################################################

init $argc $argv

Window show .
Window show .top17

main $argc $argv


